/* body.c,	 process text */

#include "vpt.h"
#include "att_table.h"
#include "prefix.h"

static Att_table	body_atts;		/* attributes for text body */

static int widowlines;		/* widow line counter */

void
doline(void)
{
	KCHAR buffer[4000], prebuff[200], c;
	register int linelen, linedone, lm_size, t;
	int left_id, right_id, center_id, mn_size;
	char cb_status, mn_status;
	
	if(getbotmarg()>line_page) terminate("Bottom margin past page bottom");
	if(gettopmarg()>getbotmarg()) terminate("Top margin past bottom margin");
	
	for( ; ; )
	{
		do_dumb_printer(&body_atts);	/* do line start fix for dummies */
		
		if(getlnum()<=gettopmarg()) dotopmargin();
		
		doeolwhens();
		
		lm_size=getleftmg(0, pageodd());		/* left marg size */
		left_id=getbothid()+getleftid()+needtempid();	/* left id size	  */
		right_id=getbothid()+getrightid()+needtemprid();/* right id size  */
		
		/* Linelength for text... */
		
		linelen=getrightmg(0, pageodd())-lm_size;
		if(numcols) linelen=(linelen-(getgutter()*(numcols-1)))/numcols;
		if(lineprefix) linelen-=lineprefixsize;
		if(linesuffix) linelen-=linesuffixsize;
		
		/* marginal note size */
		
		mn_size=(mn_status=mnotemarg(pageodd())) ? mnotesize() : 0;
		
		center_id=needcenterid();				/* center id */
		cb_status=getcbarstatus(pageodd());		/* init changebar status */
		
		if(linelen-center_id-left_id-right_id-mn_size < msperin*2)
			terminate("Total line length less than one character");
		
		linedone=fillbuff(buffer, buffer+arraysize(buffer)-2, 
			linelen-mn_size-left_id-right_id, left_id, 
			center_id, &body_atts, FMT_TEXT);
		
		/*	get the change bar status. This was done BEFORE the line was
			formatted, and now we OR in the current status. This method
			ensures that the bar will be printed in lines where the on
			and off instructions occur on the same line or on consecutive
			lines.
		*/
		
		cb_status|=getcbarstatus(pageodd());
		
		if(cb_status==1 || cb_status==3) t=docbar(lm_size/2);
		else t=0;
		
		if(!numcols) domargin(lm_size-t);
		if(mn_status==1) linedone+=printmnote(1);
		
		if(lineprefix)
		{
			memcpy(&temp_atts, &suffix_atts, sizeof(suffix_atts));
			returnstr(lineprefix);
			t=fillbuff(prebuff, prebuff+sizeof(prebuff)-2, lineprefixsize,
				0, 0, &temp_atts, FMT_PREFIX);
			printline(prebuff, &temp_atts);
			while(getkey() != INT_DELIM);
			if(t<lineprefixsize) domargin(lineprefixsize-t);
		}
		
		if(left_id) linedone+=domargin(left_id);
		
		printline(buffer, &body_atts);
		
		if(right_id && (numcols || mn_status==2 || cb_status>=2))
			linedone+=domargin(right_id);
		
		/* pad lines for cols and right margin notes */
		
		if(mn_status==2)
		{
			linedone+=output_space(linelen-linedone-mn_size);
			linedone+=printmnote(2);
		}
		
		if(numcols || linesuffix)
		{
			linedone+=output_space(linelen-linedone-mn_size);
			force_n_font();
		}
		
		if(linesuffix)
		{
			memcpy(&temp_atts, &suffix_atts, sizeof(suffix_atts));
			returnstr(linesuffix);
			t=fillbuff(prebuff, prebuff+sizeof(prebuff)-2, linesuffixsize, 
						0, 0, &temp_atts, FMT_SUFFIX);
			printline(prebuff, &temp_atts);
			while(getkey() != INT_DELIM);
			if(t<linesuffixsize) domargin(linesuffixsize-t);
		}
		
		if(cb_status>=2) docbar((linelen-linedone)+(lm_size/2));
		
		if(numcols)
		{
			setcolline();
			force_n_font();
		}
		else
		{
			if(linespace>0) do_full_line();
			partline();
			for(t=1; t<linespace && !at_page_bot(); t++)
			{
				if(numcols)
				{
					setcolline();
					force_n_font();
				}
				else do_full_line();
			}
		}
		
		updatedelayid();   /* set the delayed indent flag, if nesc. */
		
		if(at_page_bot())  do_new_pg();
		
		/*	See if we are at a CR/EOF. If we are, check the embedded
			delayed indents, then break out of the loop and return
			to the main loop--it might be that the next line is
			a command line...
		*/
		
		if(!(c=getkey()) || c==CR || c==INT_CR)
		{
			update_dytab();
			if(widowlines) pgifneed(widowlines);
			break;
		}
		else returnkey(c);  /* not CR, loop some more */
	}
}

/*	============================================================
	Set the font/attributes from a command line
*/

void
setbodyatt(KCHAR c)
{
	register int t;
	
	if(is_font(c)) body_atts.cfont=c-INT_FONT;
	
	else if(is_attroff(c))
	{
		c-=INT_ATTROFF;
		body_atts.aselect[c]=0;
		if(getasize(c)) body_atts.sizemod=1;
	}
	else if(is_attron(c))
	{
		c-=INT_ATTRON;
		body_atts.aselect[c]=1;
		if((t=getasize(c))) body_atts.sizemod=t;
	}
}

/*	============================================================
	Return a pointer to the body attribute table. Called by the
	state commands
*/

Att_table *
getbodyatts(void)
{
	return &body_atts;
}

/*	============================================================
	Find the current font. Called by 'cf' command, mnotes.
*/

int
getbodyfont(void)
{
	return body_atts.cfont;
}

/*	============================================================
	Set the widow line counter
*/

void
set_widowlines(void)
{
	skipequal();
	widowlines=getBval(0);
}

/*	============================================================
	Initialize the font for the body.
*/

void
init_body(void)
{
	body_atts.cfont=getregnum('n');
}
