/* codes.c */

#include "vpt.h"
#include "codes.h"

#define MAXCODES 1024

static u_char codebuff[MAXCODES+2];

/*	============================================================
	Send a sequence of printer codes to the output path
*/

void
docodes(Codes *s)
{
	int count;
	u_char *p;
	
	if(printpg && s)
	{
		count=s->codecount;
		p=s->codelist;
		
		while(count--) output(*p++);
	}
}

/*	============================================================
	Set a codesequence.
	
	The maximum number of permitted codes in any sequence is 1024.
	
	A pointer to the buffer must be passed. If there is an error -1 is
	returned.
*/

void
setcodes(Codes **pcodes, char *emsg)
           					/* error message text */
                        	/* ptr to existing Code struct */
{
	KCHAR c;
	int count, t;
	u_char *p;
	char msg[200];
	
	for(count=0, p=codebuff; ; )
	{
		returnkey(c=getnonb());
		
		if(!c || c==';' || c==CR) break;
		
		if(istoken(c) &&(c!=INT_GH && c!=INT_HSPACE))
			terminate("Tokens not permitted in %s sequence", emsg);
		
		if(isdelm(c))
		{
			getstring(msg,sizeof(msg)-2,emsg);
			t=strlen(msg);
		}
		
		else if(c=='%')
		{
			c=getnonb();	/* get rid of the '%' */
			t=getval(0);	/* grab value after the % */
			sprintf(msg,"%d",t);
			t=strlen(msg);
		}
		
		else
		{
			*msg=getval(0);
			t=1;
		}
		
		
		if(count+t > MAXCODES-1)
			terminate("Too many codes in %s sequence", emsg);
		
		memcpy(p,msg,t);
		count+=t;
		p+=t;
	}
	
	free(*pcodes);
	if((*pcodes=(Codes *)malloc(sizeof(Codes)+count)) == NULL)
		terminate("No memory for %s definition", emsg);
	
	(*pcodes)->codecount=count;
	memcpy( (*pcodes)->codelist, codebuff, count);
}


/*	============================================================
	Send a series of codes to the output path (PI). These are always
	sent, even if printing is off for a page range...
*/

static Codes *pi;

void
dopicodes(void)
{
	int v;
	
	v=printpg;		/* even if page range says don't */
	printpg=1;		/* print--we want init done when doing */
				/* a partial print... */
	
	skipequal();
	
	setcodes(&pi,"printer initialization");
	if(printoff==0) docodes(pi);  /* don't print if PT=OFF */
	
	flushoutput();	/* make sure it takes right away! */
	
	printpg=v;
}

