/* column.c */

#include "vpt.h"

static FILE *colpath;

typedef struct bp
{
	long	lineptr;
	u_int	lsize;
}Bp;

static Bp *buffptr;

static int maxbuflines;
static short linesaved;
static char col_num;

static int column_full_line;
static int column_half_line;
static int column_part_line;

// static function prototypes

static void docolline(int);

/*	============================================================
	Return the current line number. Note that for columns we are
	tracking the current line (not column) in addition to the lines
	in the spool buffer.
*/

int
getlnum(void)
{
	return (numcols==0) ? currline : column_full_line+currline;
}

/*	============================================================
	Return the current column being printed in.
	
	If single column, returns 0
	multiple column, returns current column (1..)
*/


int
getcurrcol(void)
{
	return (numcols==0) ? 0 : col_num;
}

/*	============================================================
	See if the current line is the last line of a page. This is
	called by doline() and do_new_page() to see if the bottom
	of a page has been reached.
*/

int
at_page_bot(void)
{
	char flag=0;
	int ln;
	
	ln=getlnum();	 /* get line number and set column number */
	
	if(!numcols)
	{
		if(ln+1>=getbotmarg()) flag++;
	}
	
	else
	{
		if((col_num==numcols && ln+1>=getbotmarg()) || (col_num>numcols))
		{
			if(numcols % linesaved) flag++;
		}
	}
	return flag;
}

/*	============================================================
	Flush the column buffer (on disk) 
*/


void
flushcols(void)
{
	int col1, gap, t;
	int oldcols=numcols;

	
	/*	ensure that there are the same number of lines avail for
		each column.
	*/
	
	while(linesaved % numcols) blankline();
	
	if(linesaved)		/* unspool */
	{
		col1=0;
		gap=linesaved/numcols;
		numcols=0;	/* force output to printer */
		
		while(linesaved>0)
		{
			domargin(getleftmg(0, pageodd()));
			for(t=0; t<oldcols; t++)
			{
				if(t) domargin(getgutter());
				docolline(col1+(gap*t));
			}
			for(t=0; t<linespace; t++) do_full_line();
			partline();
			col1++;
		}
		fix_partline();
		numcols=oldcols;
		fseek(colpath, 0L, 0);	/* rewind buffer file */
		linesaved=0;
		column_full_line=0;
		column_half_line=0;
		column_part_line=0;
		col_num=1;
	}
}

static void
docolline(int n)
{
	int t;
	Bp *p;
	
	p=&buffptr[n];
	fseek(colpath, p->lineptr, 0);
	t=p->lsize;
	while(t--) output(getc(colpath));
	p->lsize=p->lineptr=0;
	linesaved--;
}

/*	============================================================
	Set the file position pointer. This is called when a new line
	is needed in column mode. The number of lines saved is also
	bumped. This is called instead of a linefeed routine when spooling
	columns.
*/

void
setcolline(void)
{
	buffptr[++linesaved].lineptr=ftell(colpath);
	
	/* Track the number of lines relative to the page size we have in
		the buffer. This accounts for multiple line spacing as well as
		partial lines.
	*/
	
	column_full_line+=linespace;
	if(halfline_on)
	{
		column_half_line++;
		if(column_half_line>=2)
		{
			column_full_line++;
			column_half_line-=2;
		}
	}
	if(partline_space)
	{
		column_part_line+=partline_space;
		while(column_part_line>=parts_per_line)
		{
			column_full_line++;
			column_part_line-=parts_per_line;
		}
	}
	
	if(column_full_line+currline+1>=getbotmarg())
	{
		column_part_line=0;
		column_half_line=0;
		column_full_line=0;
		col_num++;
	}
}

/*	============================================================
	Buffer a single character to the buffer file. Called from output().
*/

void
colbuffer(int c)
{
	if(linesaved>maxbuflines) terminate("Column pointer overflow");
	putc(c, colpath);
	buffptr[linesaved].lsize++;
}

/*	============================================================
	set number of columns
*/

void
setncols(void)
{
	char n;
	int t;
	
	skipequal();
	if((n=getBval(1))!=numcols)
	{
		if(numcols==0)
		{
			fix_partline();	  /* flush half-line feeds */
			force_n_font();
		}
		
		if(n==1) n--;	/* set 1 column as 0 for easy checking... */
						/* numcols NEVER == 1. It'll be 0, 2, 3... */
		
		if(numcols) flushcols();	/* flush stored stuff */
		
		if(n && colpath==NULL)		/* open the scratch file */
		{
			if((colpath=tmpfile())==NULL)
				terminate("Can't open column scratch file, %s",
					 strerror(errno));
		}
		
		if(n)
		{
			if(buffptr) free(buffptr);
			maxbuflines=get_lines_page()*n;
			t=maxbuflines*sizeof(Bp);
			if((buffptr=(Bp *) malloc(t))==NULL)
				terminate("No memory for column pointers");
			linesaved=0;
			column_full_line=0;
			column_half_line=0;
			column_part_line=0;
			col_num=1;
			memset(buffptr, 0, t);
		}
		numcols=n;
	}
}


/*	============================================================
	Return flag if columns are active. All this does is check
	the linessaved variable--if linesaved is 0 then no column
	output has been done yet.
*/

int
colactive(void)
{
	return (numcols==0) ? 0 : linesaved;
}

/*	============================================================
	This error routine is called from functions which are illegal
	in column mode. Pass the function name for proper error reporting
*/

void
columnerror(char *s)
{
	terminate("%s command is incompatible with columns", s);
}
