/* crs.c */

#include "vpt.h"
#include "codes.h"

static char halfline_counter;
static int partline_counter;

static Codes *crcodes,		/* carriage return code (defaults to CR) */
			 *hlcodes,		/* codes for a single half-line feed */
			 *plcodes;		/* codes to feed 1/n line */

/*	======================================================================
	Return the number of lines per page...this is used by setncols() to
	determine the amount of storage needed to store the buffered columns
	pointers.
*/

int
get_lines_page(void)
{
	int n, t;
	
	if(partline_space)
	{
		n=(linespace * 100) + (partline_space * 100)/parts_per_line;
		t=line_page * 100;
		return t/n; 
	}
	
	else if(halfline_on)
	{
		n=line_page * 10;
		t=(linespace * 10) + 5;
		return n/t;
	}
	
	
	else return line_page/linespace;
}

/*	============================================================
	Set the carriage return codes (initialized to CR)
*/

void
setcrcodes(void)
{
	skipequal();
	
	setcodes(&crcodes,"CR code");
}

/*	Set the halfline spacing code (ch=..) */

void
sethfcodes(void)
{
	skipequal();
	
	setcodes(&hlcodes, "half-line code");
}

void
setplcodes(void)
{
	skipequal();
	setcodes(&plcodes, "part-line code");
	if(getnonb()!=';') terminate("Need parts-per-line after part-line codes");
	parts_per_line=getval(0);
}



/*	============================================================
	Set the line spacing
*/

void
setlinespace(void)
{
	KCHAR c;
	int t;
	
	if(colactive())
	{
		char buff[50];
		
		t=numcols;
		sprintf(buff, "co=1 pr[] co=%d\n", numcols);
		returnstr(buff);
		docmdline();
	}
	
	if(partline_counter || halfline_counter) fix_partline();
	
	halfline_on=0;
	partline_space=0;
	linespace=0;
	
	skipequal();
	
	/* special test required for .xx format */
	
	c=getnonb();
	returnkey(c);
	if(c!='.') linespace=getval(0);
	
	/* now see if using a '.'...parse 2nd value if nesc */
	
	if((c=getkey())=='.')
	{
		c=getkey();
		if(!f_isdigit(c)) error("Expecting digit after . in LS command");
		t=(c-'0')*10;
		c=getkey();
		if(f_isdigit(c)) t+=c-'0';
		else returnkey(c);
		
		if(t)
		{
			if(t==50) halfline_on=1;
			else
			{
				if(plcodes==NULL)
					 error("Part line spacing must be defined before LS X.XX command");
				partline_space=(t*parts_per_line)/100;
			}
		}
	}
	else returnkey(c);	/* return NOT '.' */
	
	/* don't permit partial line spacing if it is not defined */
	
	if(hlcodes==NULL && halfline_on) halfline_on=0;
	if(plcodes==NULL && partline_space) partline_space=0;
	
	/* don't permit no line spacing at all */
	
	if(halfline_on==0 && partline_space==0 && linespace==0) linespace=1;
}

/*	============================================================
	Update the half-line spacing in anticipation for a bottom
	margin or a linespacing change.
*/

void
fix_partline(void)
{
	int t;
	
	while(halfline_counter) partline();
	if(partline_counter)
	{
/*		t=partline_space;
		partline_space=parts_per_line-partline_counter;
		partline();
		partline_space=t;*/
		t=partline_space;
		partline_counter+=t;
		while(t--) docodes(plcodes);
		while(partline_counter>=parts_per_line)
		{
			currline++;
			partline_counter-=parts_per_line;
		}
	}
}

/*	============================================================
	Send the CR codes. Note that docodes() checks to see if
	printing is enabled. This is called from newline().
*/

void
do_full_line(void)
{
	docodes(crcodes);
	currline++;
}

/*	============================================================
	Send the half-line codes to the printer. This is only done
	if half-line spacing is enabled, the half-line codes are defined
	and column printing is off. This is called from doline() at the
	end of each line of body text and from fix_partline().
	
	Note that docodes() checks to see if printing is enabled.
*/

void
partline(void)
{
	int t;
	
	if(halfline_on==0 && partline_space==0) goto EXIT;
	
	if(halfline_on)
	{
		if(hlcodes)
		{
			docodes(hlcodes);
			halfline_counter++;
		}
		else if(plcodes)
		{
			t=parts_per_line/2;
			halfline_counter++;
			while(t--) docodes(plcodes);
		}
	}
	else if(partline_space && plcodes)
	{
		t=partline_space;
		partline_counter+=t;
		while(t--) docodes(plcodes);
	}
	
	/* adjust the real line counter for halflines and partlines */
	
	while(halfline_counter>=2)
	{
		currline++;
		halfline_counter-=2;
	}
	
	while(partline_counter>=parts_per_line)
	{
		currline++;
		partline_counter-=parts_per_line;
	}
	
EXIT:;
}
