/* fnotes.c,  footnote/endnote commands */

#include "vpt.h"
#include "att_table.h"

#define MAXNOTES 20

static char notefont;

static char	*notes[MAXNOTES],	/* pointers to the buffers */
		notenp[20],	   		/* n1 - before # in text */
		notens[20],			/* n2 - after # in text  */
		noteprefix[20], 	/* n3 - before # in note */
		notesuffix[20], 	/* n4 - after # in text  */
		notedelim[100], 	/* n5 - body/note delim  */
		noteformat,	   		/* format (l,c,j,r)		 */
		notespacing,		/* spacing between notes */
		internspace, 		/* spacing between note lines */
		botmargchange;  	/* flag, change has been made to botmarg */

static FILE *notefile;			/* where end notes are stored */
static char *notefilename;	 	/* name of endnote file */

static u_int noteid;			 	/* indent value for notes */

static short numnotes;		 	/* number of notes pending */

// static function prototypes

static void saveEnote(char *num);
static void saveFnote(char *num);
static void getnotestr(char *p, int s, char *e);

/*	============================================================
	Extract a footnote entry from the text stream during formatting
	and save it in the appropriate buffer.
	
	When the first footnote is encountered we set the footnote font
	to the current font.
*/


void
extractnote(void)
{
	char numbuff[30];
	static FLAG first=0;
	
	if(!first)
	{
		notefont=getbodyfont();
		first++;
	}
	
	stringnum('n', numbuff, sizeof(numbuff)-2);
	
	if(noteformat=='E') saveEnote(numbuff);
	else saveFnote(numbuff);
	
	setnumval('n', getnumval('n')+1);  /* inc the note number */
	
	returnstr(notens);	/* shove the note number into the input stream */
	returnstr(numbuff);	/* so that the body routine can include it	   */
	returnstr(notenp);
}

static void
saveEnote(char *num)
{
	int t;
	char nfilename[300], text[500];
	
	if(!notefile)	/* make sure file is open... */
	{
		if(!notefilename)
		{
			sprintf(nfilename, "vpt.enotes.%x", getpid());
			notefilename=strdup(nfilename);
		}
		if((notefile=fopen(notefilename, "w+"))==NULL)
			terminate("Can't open note spool file '%s', %s",
				notefilename, strerror(errno));
	}
	
	fputs(noteprefix, notefile);
	fputs(num, notefile);
	fputs(notesuffix, notefile);
	getstring(text, sizeof(text)-2, "endnote");
	fputs(text, notefile);
	for(t=0; t++<=notespacing; putc(CR, notefile) );
}

static void
saveFnote(char *num)
{
	KCHAR c;
	int t, linelen, m;
	Att_table temp_attrs;
	KCHAR fmtbuff[1000];
	char  text[500], *p;
	
	if(numcols) columnerror("Footnotes");
	
	/* end note, extract the note and buffer it */
	
	sprintf(text, "%s%s%s", noteprefix, num, notesuffix);
	t=sizeof(text)-strlen(text)-2;
	incomment++;
	getdstring(strend(text), t, "footnote");
	incomment--;
	
	/*	convert any real CRs in the footnote to SPACE...only really
		important when in line adjust mode...stuff after a CR will
		not be printed otherwise. Note we start at the end-3 since
		the footnote data has a CR and DELIM appended to it.
	*/
	
	for(p=strend(text)-3; p>text; p--)
	{
		if(*p==CR) *p=SPACE;
	}
	
	/* calculate the number of lines this note will take.  */
	
	m=addmacro(text);
	pushkbuff();
	
	/* pseudo format till an EOL */
	
	memset(&temp_attrs, 0, sizeof(temp_attrs));
	temp_attrs.cfont=notefont;
	t=pageodd();
	linelen=getrightmg('n', t)-getleftmg('n', t);
	for(t=0, noteid=0 ; ; )
	{
		fillbuff(fmtbuff, fmtbuff+arraysize(fmtbuff)-2, linelen-noteid, 				noteid, 0, &temp_attrs, FMT_NOTE);
		t++;
		if((c=getkey())==CR) break;
		t+=(internspace-1);
		returnkey(c);
	}
	t+=notespacing;
	if(numnotes==0) t+=3;
	while((c=getkey())!=INT_DELIM);  /* flush input stream */
	pullkbuff();
	killmacro(m);
	
	/*	'T' will equal the number of lines needed for this note. If there
		is no room for this note we force a new page. Note that we have to
		increment 'T' for the delim margin if any notes were pending--newpage
		will set numnotes to 0 and we'll need the delim margin on the next
		page...but if numnotes was 0 then the delim margin was already
		allocated above.
	*/
	
	if(currline+t>=getbotmarg())  /* if no room on page for note */
	{
		if(numnotes!=0) t+=3;
		do_new_pg();				/* force new page, print notes */
		dotopmargin();				/* top margin */
	}
	
	decbotmarg(t);
	botmargchange+=t;
	
	if(numnotes>=MAXNOTES-1)  terminate("Too many footnotes pending");
	
	notes[numnotes++]=strdup(text);
}


/*	============================================================
	Advise if footnotes are pending on this page. This is called
	by setbotmarg() to see if the it is okay to change the setting.
*/

int
fnotepending(void)
{
	return numnotes;
}

/*	============================================================
	Print the stored footnotes in the bottom margin.
*/

void
printfootnotes(void)
{
	KCHAR buff[1000];
	KCHAR c;
	int n, t, m;
	int linelen, marg;
	
	Att_table temp_attrs;
	
	memset(&temp_attrs, 0, sizeof(temp_attrs));
	temp_attrs.cfont=notefont;
	
	if(numnotes)
	{
		pushkbuff();
		marg=getleftmg('n', pageodd());
		linelen=getrightmg('n', pageodd())-marg;
		
		/*	Do the note/text margin and print the separater
			string if specified. A total of three lines will
			always be done.
		*/
		
		do_full_line();
		if(*notedelim)
		{
			m=addmacro(notedelim);
			memset(&temp_attrs, 0, sizeof(temp_attrs));
			temp_attrs.cfont=notefont;
			fillbuff(buff, buff+sizeof(buff)-2, linelen, 0, 0,
				&temp_attrs, FMT_NOTE);
			domargin(marg);
			printline(buff, &temp_attrs);
			while((c=getkey())!=INT_DELIM);  /* clean up input stream */
			killmacro(m);
		}
		do_full_line();
		do_full_line();
		
		/* Do the footnote text and return the buffer memory */
		
		memset(&temp_attrs, 0, sizeof(temp_attrs));
		temp_attrs.cfont=notefont;
		
		for(n=0; n<numnotes; n++)
		{
			m=addmacro(notes[n]);
			for(noteid=0; ; )
			{
				marg=getleftmg('n', pageodd())+noteid;
				linelen=getrightmg('n', pageodd())-marg;
				fillbuff(buff, buff+sizeof(buff)-2, linelen, noteid,
							0, &temp_attrs, FMT_NOTE);
				domargin(marg);
				printline(buff, &temp_attrs);
				do_full_line();
				if((c=getkey())==CR)				/* end of note  */
				{
					while((c=getkey())!=INT_DELIM);	  /* clean stream */
					break;
				}
				else returnkey(c);
				for(t=internspace-1; t--; do_full_line());
			}
			killmacro(m);
			free(notes[n]);
			notes[n]=0;
			for(t=0; t++<notespacing; do_full_line()); /* internote space */
		}
		incbotmarg(botmargchange);		 /* reset original bottom margin */
		numnotes=botmargchange=0;
		pullkbuff();
		notefont=temp_attrs.cfont;
	}
}

/*	============================================================
	Set the number strings.
	
	N1 - inserted before the footnote number in the text
	N2 - inserted after then footnote number in the text
	N3 - inserted before the number in the reference
	N4 - inserted after the number in the reference
	N5 - printed on the first line for the area separating the text/notes
*/

void
setnum1(void)
{
	getnotestr(notenp, sizeof(notenp)-2, "N1");
}

void
setnum2(void)
{
	getnotestr(notens, sizeof(notens)-2, "N2");
}

void
setnum3(void)
{
	getnotestr(noteprefix, sizeof(noteprefix)-2, "N3");
}

void
setnum4(void)
{
	getnotestr(notesuffix, sizeof(notesuffix)-2, "N4");
}

void
setnum5(void)
{
	skipequal();
	getdstring(notedelim, sizeof(notedelim)-2, "N5 string");
}

static void
getnotestr(char *p, int s, char *e)
{
	char emsg[30];

	sprintf(emsg, "%s string", e);

	skipequal();
	getstring(p, s, emsg);
}


/*	============================================================
	set the format for note 
	
	'E'==endnotes, anything else==footnotes
*/

void
setnotefm(void)
{
	KCHAR c;
	char fname[300];
	
	skipequal();
	
	if((c=f_toupper(getkey()))=='E' || c=='F')
	{
		if((notefile && c=='F') || (numnotes && c=='E'))
			error("Can't change footnote format while notes are active");
		
		noteformat=c;
	}
	else error("Footnote formats are 'E' and 'F'");
	
	/* if setting format to endnotes we can specify an optional filename */
	
	if(noteformat=='E')
	{
		if((c=getkey())=='=')
		{
			if(notefilename) free(notefilename);
			getstring(fname, sizeof(fname)-2, "endnote filename");
			notefilename=strdup(fname);
		}
		else returnkey(c);
	}
}

/*	============================================================
	Print the stored endnotes
*/

void
printnotes(void)
{
	KCHAR c;
	
	if(!notefile) error("No endnotes to print");
	
	fclose(notefile);				/* close the existing file */
	notefile=NULL;
	setnumval('n', 1);				 /* reset the number */
	while((c=getkey()) && c!=CR);	/* flush rest of command line */
	
	/* include file to print it */
	
	if(!newfile(notefilename))
		terminate("Can't open notefile '%s'", notefilename);
	
	returnkey(CR); /* so we know the command line is finished */
}

/*	============================================================
	set the note spacing
	
	.ns=space between notes [,space between note lines]
*/

void
setnotesp(void)
{
	if(numnotes)
		error("Can't change note spacing while notes pending");
	
	skipequal();
	notespacing=getBval(0);
	if(comma_ck()) internspace=getBval(1);
}


/*	============================================================
	Set the delayed indent for a footnote. Note that only one \id
	is permitted per note. If a footnote has more than one \id in
	it then only the last one encountered is used. This is called
	from fillbuff().
*/

void
setnoteid(int offset)
{
	noteid=offset;
}

/*	======================================================================
	Adjust the marginal note indent after a MS=
*/

void
ms_fnote(u_int n)
{
	noteid=ms_adjust(noteid, n);
}


void
delnotefile(void)
{
	if(notefile)
	{
		fclose(notefile);
		unlink(notefilename);
	}
}
