/* font.c

   Functions to set up font tables and extract information
*/

#include "vpt.h"
#include "codes.h"
#include "att_table.h"
#include "font.h"

static Fontdata *fontptrs[NUM_FONTS+1];

/* ============================================================
   Tokenize an embedded font
*/

void
tokfont(void)
{
	int t;
	
	if((t=getregnum(getkey()))==-1) error("Unknown embedded font name");
	returnkey(INT_FONT+t);
}

/* ============================================================
   Get the size of the character in microspaces
*/

int
getcsize(KCHAR c, Att_table *att)
          /* character $20..$ff or token */
               
{
	Fontdata *p;
	int csize;
	int t;
	
	if(c==INT_HSPACE) c=' ';

	if(istoken(c))
	{
		if(is_dkey(c)) csize=getdksize(c-INT_DKEY, att);
		else if(c==INT_SSPACE) csize=smallspacesize();
		else csize=0;
		goto exit;
	}
	
	if(translate) c=translatechar(c);
	
	if((p=fontptrs[(int)att->cfont])) csize=p->f_sizes[c];
	else csize=msperin;
	
	if((t=att->sizemod)>1) csize*=t;
	
exit:
	return csize;
}

/*	============================================================
	Set a font sequence and the width table.
	
	Fn=<on codes>; <off codes>; <sizes>
	
	<sizes> == [*multiplier]; <list of sizes>
		or [\font]
	
	If sizes are specified as \fn then the size table from the specified
	font is copied. This is great for printers which have several proportional
	fonts with identical width tables.
*/

void
setfont(void)
{
	int t;
	FSIZETY mult, v;
	FSIZETY *f;
	KCHAR c;
	Fontdata *p;
	int fn;
	
	if((t=getregnum(getkey()))==-1) error("Illegal font name");
	fn=t;
	
	if((p=fontptrs[t])==NULL)
	{
		if((p=fontptrs[t]=(Fontdata *) calloc(sizeof(Fontdata),
			1))==NULL) terminate("Font data buffer overflow");
	}
	
	skipequal();
	
	returnkey(c=getnonb());
	if(isdelm(c))
	{
		set_soft_font(fn, p);
		goto EXIT;
	}
	
	setcodes(&p->f_oncodes,"font on-code");
	
	if((c=getnonb())!=';') terminate("';' expected after font on codes");
	
	setcodes(&p->f_offcodes,"font off-code");
	
	if((c=getnonb())!=';') terminate("';' expected after font off codes");
	
	/* could be a font copy same size table.  */
	
	if(is_font(c=getnonb()))
	{
		t=c-INT_FONT;
		if(fontptrs[t]==0 || t==fn)
			terminate("Can't assign widths from font %c", ascregnum(t));
		memcpy(p->f_sizes, fontptrs[t]->f_sizes, sizeof(FSIZETY)*NUMFSIZES);
	}
	
	/* not a font copy, read a new table */
	
	else
	{
		if(c=='*') mult=getval(0);
		else
		{
			returnkey(c);
			mult=1;
		}
		
		/* get first size and set entire table to it */
		
		returnkey(getnonb());
		v=(FSIZETY)getval(0)*mult;
		for(t=0,  f=p->f_sizes; t<NUMFSIZES; *f++=v, t++);
		
		for(t=' '+1, f=&p->f_sizes[' '+1] ; ; )
		{
			returnkey(c=getnonb());
			if(!c || c==';' || c==CR) break;
			*f++=(FSIZETY)getval(0)*mult;
			if(++t>NUMFSIZES)
				terminate("Too many size entries for font definition");
		}
	}
EXIT:;
}


/* ============================================================
   Send the font codes. We first load 'f' with the address of
   the font data package. If this is 0 then the font has not
   been defined and we do nothing.
   
   Next we load 'p' and 'count' with the on/off data and count
   
   Finally, the data is send to output.
*/

void
sendFcodes(int n, int onoff)
{
	Fontdata *f;
	
	if((f=fontptrs[n]))
	{
		/* If a font name is defined in the font structure it means
			that we a soft font which has not yet been loaded. So,
			we load it now and the delete the name.
		*/
		
		if(f->fname)
		{
			load_font(f->fname, n);
			free(f->fname);
			f->fname=NULL;
		}
	
		/* Now, send the codes...of course, it is not nesc. if we are
			not printing this page.
		*/
	
		if(printpg)
		{
			if(onoff==0)	docodes(f->f_offcodes);
			else			docodes(f->f_oncodes);
		}
	}
}

/*	======================================================================
	See if any fonts defined...this is called after a MS command. If any
	fonts have been defined we exit....
*/


void
ms_font(u_int n)
{
	int t;
	
	for(t=0; t<arraysize(fontptrs); t++)
	{
		if(fontptrs[t]) terminate("MS=%d follows a font definition",n);
	}
}
