/* index.c */

#include "vpt.h"

static FILE *auxfile, *tocfile, *inxfile;

static char indexdelim[50];

static char *inxnameptr;

static void doindex(FILE *);


/*	============================================================
	Set the index filename. This is just a pointer to the name
	in the arg list. This is called from vpt.c initialization
*/

void
setinxname(char *p)
{
	inxnameptr=p;
	if(strlen(p)>150) p[150]=0;
}

/*	============================================================
	Set the index delimiter character
*/

void
setindexdlm(void)
{
	skipequal();
	getstring(indexdelim, sizeof(indexdelim)-2, 
				"index delimiter string");
	if(*indexdelim=='\0') *indexdelim=CR;
}

/*	============================================================
	Open the output files for index, table of contents and aux.
   	
	These routines get called by the .OX .OC and .OA commands
*/

void
openauxf(void)  /* aux output */
{
	char fname[200];
	char onoff=getonoff();
	static char *mode="w";
	
	if(onoff==0)
	{
		if(auxfile) fclose(auxfile);
		auxfile=0;
		mode="a";
	}
	else
	{
		if(!auxfile)
		{
			sprintf(fname, "%s.%s", inxnameptr, "aux");
			if((auxfile=fopen(fname, mode))==NULL)
				terminate("Can't open auxillary-output file '%s'", fname);
		}
	}
}

void
opentocf(void)	 /* table contents */
{
	char fname[200];
	char onoff=getonoff();
	static char *mode="w";
	
	if(onoff==0)
	{
		if(tocfile) fclose(tocfile);
		tocfile=0;
		mode="a";
	}
	else
	{
		if(!tocfile)
		{
			sprintf(fname, "%s.%s", inxnameptr, "toc");
			if((tocfile=fopen(fname, mode))==NULL)
				terminate("Can't open table-of-contents output file, '%s'",
							 fname);
		}
	}
}


void
openinxf(void)  /* index */
{
	char fname[200];
	char onoff=getonoff();
	char *mode="w";
	
	if(onoff==0)
	{
		if(inxfile) fclose(inxfile);
		inxfile=0;
		mode="a";
	}
	else
	{
		if(!inxfile)
		{
			sprintf(fname, "%s.%s", inxnameptr, "inx");
			if((inxfile=fopen(fname, mode))==NULL)
				terminate("Can't open index output file '%s'", fname);
		}
	}
}

/*	============================================================
	Output an encountered reference to the file
*/

void
expinx(void)
{
	doindex(inxfile);
}

void
exptoc(void)
{
	doindex(tocfile);
}

void
expaux(void)
{
	doindex(auxfile);
}

static void
doindex(FILE *outfile)
{
	char buffer[530];
	
	getstring(buffer, sizeof(buffer)-30, "index buffer");
	
	if(!doingcmd) returnstr(buffer);
	
	if(outfile)
		fprintf(outfile, "%s%s%d\n", buffer, indexdelim, getnumval('P'));
}

