/* hpvtbl.c */

#include "vpt.h"
#include "ljfont.h"
#include "codes.h"
#include "font.h" 

#define MAXFONTNAME 100

static char *soft_font_dir;		/* Pointer to the soft font directory */

static int get_eseq(char *ebuf, int esiz, FILE *fptr);
static void parse_eseq(char *eseq, int *etyp, int *eval);
static void make_soft_font(short int *widths, char *fontname);

/* ======================================================================
	set the directory name for soft fonts
*/

void
set_soft_font_dir(void)
{
	char buff[200];
	
	if(soft_font_dir)
	{
		free(soft_font_dir);
		soft_font_dir=NULL;
	}
	skipequal();
	getstring(buff, sizeof(buff)-2, "Soft font directory");
	soft_font_dir=strdup(buff);	
}

/* ======================================================================
	Set a soft font. We assume that we know the laser print on/off codes...
	for now they are hard-coded.
	
	fn	font number 0..9, a..z
	ftable	ptr to font information structure
*/

void
set_soft_font(int fn, Fontdata *ftable)
{
	int t;
	char *fontpath;
	char fontname[MAXFONTNAME];
	char temp[MAXFONTNAME];
	FSIZETY widths[NUMFSIZES];
	
	getstring(fontname, MAXFONTNAME-1, "Soft font filename too long");
	
	asprintf(&fontpath, "%s/%s", soft_font_dir ? soft_font_dir : ".", fontname);
	if(!fontpath)  terminate("No memory to process soft font");
	
	make_soft_font(widths, fontpath);
	
	/*	Adjust hard-sizes from the font file to the current internal
		width being used.
	*/
	
	for(t=32; t<NUMFSIZES; t++) widths[t]=widths[t]*msperin/120;
	
	/* copy to font desc */
	
	memcpy(ftable->f_sizes, widths, sizeof(widths));
	
	/* Set the font on codes for this font */
	
	sprintf(temp, "27 '( %%%d 'X", fn);
	returnstr(temp);
	setcodes(&ftable->f_oncodes, "Soft font on codes");
	
	/*	Save the font name...the font will be transfered to the printer
		when it is first used.
	*/
	
	free(ftable->fname);
	ftable->fname=strdup(fontpath);	/* save name for later uploading */
	
	free(fontpath);
}

/* ======================================================================

	The following is a code modified from the LaserJet Book. All it does
	is parse a font file and extract the character sizes.
*/

#define MAX_ESEQ 80

/* ------------------------------------------------------------
	Gather the next esc sequence
*/

static int
get_eseq(char *ebuf, int esiz, FILE *fptr)
{
	int i, c;
	
	for( ; ; )
	{
		c=getc(fptr);
		if(c==-1) return 0;
		if(c==0x1b) break;
	}
	i=0;
	do
	{
		c=getc(fptr);
		if(c==-1) terminate("Sync error reading soft font");
		if(i==esiz) terminate("Buffer overflow reading soft font");
		ebuf[i++]=c;
	}
	while(!IS_TERM_CHR(c));
	
	return 1;
}

/* ------------------------------------------------------------
	Parse an esc sequence
*/

static void
parse_eseq(char *eseq, int *etyp, int *eval)
{
	int i;
	char pchar, gchar, tchar;
	
	*etyp=eUNKNOWN;
	*eval=0;
	
	pchar=eseq[0];
	gchar=eseq[1];
	*eval=atoi(eseq+2);
	
	i=2;
	while((!IS_TERM_CHR(eseq[i]))&&(i<MAX_ESEQ)) i++;
	tchar=eseq[i];
	
	if((pchar==')') && (gchar=='s') && (tchar=='W')) *etyp=eFONTDESC;
	else if((pchar=='(') && (gchar=='s') && (tchar=='W')) *etyp=eCHARDESC;
	else if((pchar=='*') && (gchar=='c') && (tchar=='E')) *etyp=eCHARCODE;
}

/* ------------------------------------------------------------
	Create a soft font table for the specified file
*/

static void
make_soft_font(short int *widths, char *fontname)
{
	FILE *fptr;
	int t, elast, cur_ch;
	int etype, eval;
	font_desc *fd=NULL;
	char_desc cd;
	char eseq[MAX_ESEQ];
	
	cur_ch = -1;
	elast = eUNKNOWN;
	
	if((fptr=fopen(fontname, "r"))==NULL)
		terminate("Can't open soft font '%s'", fontname);

	while(get_eseq(eseq, MAX_ESEQ, fptr))
	{
		parse_eseq(eseq, &etype, &eval);
		switch(etype)
		{
			case eFONTDESC:
				free(fd);
				fd=(font_desc*)malloc(eval);
				if(fd==NULL) terminate("Memory failure reading soft font");
				memset(fd, 0, eval);
				fread(fd, eval, 1, fptr);
				if(fd->fd_fmt!=fBITMAP)
					terminate("Soft font not bitmapped--cannot use");
				
				/* set all entries to default size (space) */
				
				for(t=32; t<NUMFSIZES; widths[t++]=REV_WRD(fd->pitch));
				break;
				
			case eCHARDESC:
				if(elast==eCHARDESC) fseek(fptr, eval, 1);
				else
				{
					fread(&cd, sizeof(char_desc), 1, fptr);
					fseek(fptr, eval-sizeof(char_desc), 1);
					widths[cur_ch]=REV_WRD(cd.delta_x);
				}
				break;
				
			case eCHARCODE:
				cur_ch=eval;
				break;
				
			default:
				break;
		}
		elast=etype;
	}
	free(fd);
	
	fclose(fptr);

} 

