/*	ljfont.h
	HP Soft Font types header file
	From HP LaserJet Programming, Binstock, Babcock, Luse pp.90
*/

/* Structures within a HP font file */

typedef struct _font_desc
{
	u_short	fd_size;	/* desc. size */
	u_char	fd_fmt;		/* desc. format */
	u_char	font_type;	/* font type */
	u_char	style_msb;	/* style word msb */
	char	resv_1;
	u_short	bl_dist;	/* baseline distance */
	u_short	cell_wid;	/* cell width */
	u_short	cell_hgt;	/* cell height */
	u_char	orient;		/* 0==por, 1==land */
	u_char	spacing;	/* 0==fixed, 1=prop */
	u_short	sym_set;	/* symbol set */
	u_short	pitch;		/* pitch */
	u_short	height;		/* height */
	u_short	xheight;	/* x height */
	char	wid_typ;	/* width type */
	u_char	style_lsb;	/* style word lsb */
	char	stk_wgt;	/* stroke weight */
	u_char	typeface;	/* typeface */
	u_char	vendor;		/* vendor version */
	u_char	serif;		/* serif style */
	u_char	quality;
	char	placement;
	char	ul_dist;	/* underline distance */
	u_char	ul_hgt;		/* underline height */
	u_short	txt_hgt;	/* text height */
	u_short	txt_wid;	/* text width */
	u_short	min_ch;		/* 1st ADE in font */
	u_short	max_ch;		/* last ADE in font */
	u_char	pitch_ext;	/* pitch extended */
	u_char	height_ext;	/* height extended */
	u_short	cap_hgt;	/* cap letter hgt */
	u_long	font_id;	/* font ID code */
	char	font_name[16];	/* font name */
		/* vendor independent data follows... */
}font_desc;

/* character descriptor */

typedef struct _char_desc
{
	u_char	format;		/* LJII=4 */
	char	continued;	/* 0=cd, 1=data */
	u_char	cd_size;	/* des+data size */
	u_char	cd_class;	/* LJII=1 */
	u_char	orient;		/* 0=por, 1=land */
	char	resv_1;
	short	left_ofs;	/* left offset */
	short	top_ofs;	/* top offset */
	u_short	char_wid;	/* char wid in dots */
	u_short	char_hgt;	/* char height in dots */
	short	delta_x;	/* cur pos change */
		/* character bit map follows */
}char_desc;

/* ========================================= */
/* These define structures used in the book code */

typedef int width_tbl[256];

typedef struct _font_hdr
{
	char name[18];	/* name of font */
	short family;	/* typeface family */
	short orient;	/* port or landscape */
	short spacing;	/* prop or fixed */
	short posture;	/* upright or italic */
	short weight;	/* light, med, bold, etc. */
	short pitch;	/* default width */
	short height;	/* default leading */
	short bldist;	/* cell top to baseline */
	width_tbl wtbl;	/* array of character widths */
}
font_hdr;

typedef struct _string_metric
{
	double	width;		/* full width, inches */
	double	height;		/* full height, inches */
	double	nb_width;	/* width excluding blanks */
	double	ascent;		/* max ascent */
	double	descent;	/* max descent */
}
string_metric;

/* ============================= */
/* Macros */

/* reverse 2 bytes in a word... */

#define REV_WRD(w)  (((w>>8)&0xff)|(w<<8))

/* a 2char, non-parm seq begins with... */

#define IS_SMPL_SEQ(c) (((c>=48) && <c<=126)) ? 1 : 0)

/* a parm escape seq begins with.. */

#define IS_PARM_SEQ(c) (((c>=32) && (c<=47)) ? 1 : 0)

/* a parm char is form this range */

#define IS_PARM_CHR(c)	(((c>=96)&&(c<=126))?1:0)

/* termination char */

#define IS_TERM_CHR(c) (((c>=64) && (c<=94)) ? 1 : 0)

/* =============== */
/* constants */

/* escape sequence ids   */

#define eUNKNOWN	0
#define eFONTDESC	1
#define eCHARDESC	2
#define eCHARCODE	3

/* units */

#define uDPI	300
#define uQDPI	1200

/* spacing */

#define sFIXED	0
#define sPROPORTIONAL 1

/* orientation */

#define oPORTRAIT	0
#define oLANDSCAPE	1

/* font formats */

#define fBITMAP		0
#define fSCALABLE	10

/* eof */
