/* makevinx.c  -- standalone program to process vpt.inx files

	Version 1  ??/??  1992
		original version
		
	Version 2  Jan 1, 1993
		- Changed allocation to use binary tree. No limit on number of
		entires. No need for qsort() routine.
		- Fixed bug which did not add new entries to list.
		
	Version 3 April 27, 1997
		- asciified for linux
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#define strend(s) (s+strlen(s))

extern int errno;

typedef struct _entry
{
	char *e_ent;
	char *e_num;
	struct _entry *e_left;
	struct _entry *e_right;
}
tree_entry;

tree_entry *elist=NULL;

char lasthead, text[220], number[20], *save_data();

FILE *outfile, *infile;

// function prototypes

char *cgets(char *, int, FILE *);
char *save_data(char *);
tree_entry *add_new_entry(char *, char *);
void print_tree(tree_entry *);
void usage(void);
void terminate(char *, ...);

#define NUMDELIM 2

int
main(int argc, char *argv[])
{
	char *p;
	int cmp;

	tree_entry *ptr;

	/* process the command line args. */

	if(argc>3 || argc<2 || argv[1][0]=='-') usage();

	if((infile=fopen(argv[1],"r"))==NULL)
		terminate("Can't open '%s' for input, %s", argv[1], strerror(errno));

	if(argc<3)
	{
		sprintf(text,"%s.txt",argv[1]);
		argv[2]=text;
	}

	if((outfile=fopen(argv[2],"w"))==NULL)
		terminate("Can't open '%s' for output, &s", argv[2], strerror(errno));

	/* files open, read the data */

	while(!feof(infile))
	{
		cgets(text,sizeof(text),infile);	/* get the entry */
		*text=toupper(*text);			/* force 1st char uppercase */
		cgets(number+1,sizeof(number),infile);	/* get the page number */
		*number=NUMDELIM;
		p=strend(number);
		*p++=NUMDELIM;
		*p='\0';

		/* this adds the root to the linked list...only once */

		if(elist==NULL)
		{
			elist=add_new_entry(text,number);
			continue;
		}

		/* Check the entries already in memory to see if this is a duplicate.
			If found, we append the new page number to the existing one by
			copying to a new buffer.
		*/

		ptr=elist;
		for(;;)
		{
			if((cmp=strcasecmp(ptr->e_ent,text))==0)	/* duplicate entry? */
			{
				if(strstr(ptr->e_num, number))		/* dup page num? */
					break;				/* yes, exit.... */

				p=malloc(strlen(ptr->e_num)+strlen(number)+1);
				if(!p) terminate("Out of memory");
				sprintf(p,"%s%s",ptr->e_num,number);	/* save new str */
				free(ptr->e_num);			/* free old buff */
				ptr->e_num=p;
				break;
			}
			if(cmp<0)
			{
				if(ptr->e_left) ptr=ptr->e_left;
				else
				{
					ptr->e_left=add_new_entry(text,number);
					break;
				}
			}
			if(cmp>0)
			{
				if(ptr->e_right) ptr=ptr->e_right;
				else
				{
					ptr->e_right=add_new_entry(text,number);
					break;
				}
			}
		}
	}

	/* Now create a vpt text file. */

	fprintf(outfile,".CO=2");	 /* set to 2 column mode */

	lasthead=0;
	print_tree(elist);

	fprintf(outfile,".CO=1\n");	  /* end of file...force unbalanced cols */
}


/* ============================================================
	Walk the tree and print entries in order. This is a recursive tree
	printing routine...needs lots of stack!
*/

void
print_tree(tree_entry *e)
{
	char *p;

	if(e)
	{
		/* find lowest entry in this sub-tree first */

		print_tree(e->e_right);

		/* print new header line */

		if(*(e->e_ent)>lasthead)
		{
			lasthead=*(e->e_ent);
			fprintf(outfile,"\n.NL=4\n\\E*%c\n\n",lasthead);
		}

		/* print an entry...first the entry */

		fputs(e->e_ent,outfile);
		putc(' ',outfile);

		/* now the page numbers...delete the delimiters and insert
				commas and spaces...
		*/

		p=e->e_num;
		while(*p)
		{
			while(*p==NUMDELIM) p++;
			while(*p && *p!=NUMDELIM) putc(*p++,outfile);
			while(*p==NUMDELIM) p++;
			if(*p) fputs(", ",outfile);
		}
		putc('\n',outfile);

		/* do rest of the left tree */

		print_tree(e->e_left);
	}
}


/* ============================================================
   Error handler
*/

void
terminate(char *s, ...)
{
	va_list ap;
	
	va_start(ap, s);
	vfprintf(stderr, s, ap);
	va_end(ap);
	fputc('\n', stderr);
	exit(1);
}


/* ============================================================
   Read a string. Convert trailing \n to NULL and delete trailing
   blanks.
*/

char *
cgets(char *s, int maxlen, FILE *f)
{
	char *p;

	if(!(p=fgets(s,maxlen,f))) return p;

	p=strend(s)-1;
	if(*p=='\n') *p--=0;			   /* strip trailing CR */
	while(*p==' ' && p>s)  *p--='\0';  /* strip trailing spaces */

	return s;
}

/* ============================================================
   Usage message..
*/

void
usage(void)
{
	fprintf(stderr, "\nMakevinx, processes Vprint index files\nUsage: makevinx infile [outfile]\n\n");
	exit(1);
}

/* ============================================================
	Add a new node. We pass pointers to the keyword and number string.
	A pointer to the node is returned.
*/

tree_entry *
add_new_entry(char *t, char *n)
{
	tree_entry *ptr;

	ptr=calloc(1,sizeof(tree_entry));

	if(!ptr) terminate("No memory to add entry");
	ptr->e_ent=save_data(t);
	ptr->e_num=save_data(n);

	return ptr;
}

/* ============================================================
	Save a string. This will abort if there is not enuf memory
	to save.
*/

char *
save_data(char *s)
{
	char *p;

	p=malloc(strlen(s)+1);  /* get memory */
	if(p) strcpy(p,s);		/* if success, stuff string */
	else terminate("No memory to save data");

	return p;
}

