/* makevtoc.c  -- standalone program to process vpt.toc files */

/* asciified for linux, 97/04/27 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#define strend(s) (s+strlen(s))

extern int errno;

// function prototyes

void usage(void);
char *cgets(char *, int, FILE *);
void terminate(char *, ...);

int
main(int argc, char *argv[])
{
	char *p, c;
	int t, len;
	FILE *outfile, *infile;
	char text[220], number[20];

	/* process the command line args. */

	if(argc>3 || argc<2 || argv[1][0]=='-') usage();

	if((infile=fopen(argv[1],"r"))==NULL)
		terminate("Can't open '%s' for input",argv[1]);

	if(argc<3)
	{
		sprintf(text,"%s.txt",argv[1]);
		argv[2]=text;
	}

	if((outfile=fopen(argv[2],"w"))==NULL)
		terminate("Can't open '%s' for output",argv[2]);

	fprintf(outfile,".SV LF=C\n");
	fprintf(outfile,"Contents\n");
	fprintf(outfile,".SP3 LF=L IB=5 #P=1 F#P=r TF='.\n");

	/* files open, read the data */

	while(!feof(infile))
	{
		cgets(text,sizeof(text),infile);	   /* get the entry */
		cgets(number,sizeof(number),infile);   /* get the page number */

		if(*text=='\0') fprintf(outfile,"\n");
		else fprintf(outfile,"%s\\tw\%s\n",text,number);
	}
	fprintf(outfile,".IB=0 SR PG");		  /* end of file */
}

/* ============================================================
   Error handler
*/

void
terminate(char *s, ...)
{
	va_list ap;
	
	va_start(ap, s);
	vfprintf(stderr, s, ap);
	va_end(ap);
	fputc('\n', stderr);
	exit(1);
}

/* ============================================================
   Read a string. Convert trailing \n to NULL and delete trailing
   blanks.
*/

char *
cgets(char *s, int maxlen, FILE *f)
{
	char *p;

	if(!(p=fgets(s,maxlen,f))) return p;

	p=strend(s)-1;
	if(*p=='\n') *p--=0;			   /* strip trailing CR */
	while(*p==' ' && p>s)  *p--='\0';  /* strip trailing spaces */

	return s;
}

/* ============================================================
   Usage message..
*/

void
usage(void)
{
	fprintf(stderr, "\nMakevtoc, processes Vprint table of contents files\nUsage: makevtoc infile [outfile]\n\n");
	exit(1);
}
