/* page.c */

#include "vpt.h"
#include "codes.h"

#define mult10(x) ((x<<3)+(x<<1))
#define MAXRANGES 20

static struct			/* a page range */
{
	u_int	start,
		end;
}ranges[MAXRANGES];

static int oe_range;

static int numranges;	/* number of page ranges set */

static Codes *pb;		/* page bottom codes */

static char *digtoval(char *s, u_int *val, char *orig);


/*	============================================================
	Set a printing range. This is called from the command line parser.
   	
	-p=n1,n2  sets range to n1...n2
	-p=n1-	  sets range to n1 to eof
	-p=n1	  sets range to n1 only
	-p=-n1	  sets range to 0 to n1
	-p=O	  print odd pages only
	-p=E	  print even pages only
	-p=O,10-15 print pages 11,13,15
   	
   	several ranges can be concatenated with ','s
   	
   	-p=n1-n2,n3,n5-
   	
   	Up to 20 ranges can be setup.
*/

void
setpagerange(char *s)
{
	u_int n1, n2;
	char *p=s;
	char c;
	
	for( ; ; )
	{
		n1=n2=0;
		
		if(numranges>=MAXRANGES)
			terminate("Too many page ranges specified '%s'", s);
		
		c=*p;
		
		if(c=='o' || c=='O')	  /* set odd page only flag */
		{
			oe_range=1;
			p++;
			goto cont;
		}
		
		if(c=='e' || c=='E')	  /* set even page only flag */
		{
			oe_range=2;
			p++;
			goto cont;
		}
		
		if(c=='-') p=digtoval(++p, &n2, s);		  /* range = -n1 */
		
		else
		{
			p=digtoval(p, &n1, s);		/* get n1  */
			if(!(*p) || *p==',') n2=n1;	/* range= ('n1,' or ,n1,) */
			else
			{
				if(*(p++)!='-')
					terminate("'-' expected in page range '%s'", s);
				if(!(*p)) n2=0xffffffff;	/* range = n1- */
				else p=digtoval(p, &n2, s);	/* range = n1-n2 */
			}
		}
		
		if(n2<n1) terminate("Illegal page range '%s'", s);
		
		ranges[numranges].start=n1;
		ranges[numranges++].end=n2;
		
cont:
		while(*p==' ') p++;
		if(*p==',')			   /* get another page range */
		{
			p++;
			continue;
		}
		if(!(*p)) break;
		else terminate("Illegal page range character '%c' in '%s'", *p, s);
	}
}

static char *
digtoval(char *s, u_int *val, char *orig)
{
	char c;
	char *p=s;
	u_char digs=0;
	
	*val=0;
	while(*p==' ') p++;
	while((c=*p)>='0' && c<='9')
	{
		*val=mult10(*val)+(c-'0');
		p++;
		digs++;
	}
	while(*p==' ') p++;
	if(!digs) terminate("Numeric value expected in page range '%s'", orig);
	return p;
}

/*	============================================================
	Return 1 if on odd page, 0 if even
*/

int
pageodd(void)
{
	return getnumval('P') & 1;
}

/*	============================================================
	Start a new column
*/

void
startcol(void)
{
	int t;
	
	if(numcols<2) do_new_pg();	   /* not doing column, do new page */
	
	else
	{
		if(getcurrcol()==numcols) do_new_pg();  /* in last col, do new page */
		
		else	  /* skip to bottom of current column */
		{
			/*if((t=getbotmarg()-getlnum()-1)>0) while(t--) blankline();*/
			t=getcurrcol();
			while(t==getcurrcol()) blankline();
		}
	}
}

/*	============================================================
	Start a new page
	
	.pg - start new page
	.pge - start new even page
	.pgo - start new odd page
*/

void
startpage(void)
{
	char c;
	
	/*	A new page is started if we are not already at the top of a page.
		If doing columns we have to check that we are not on line 1 of
		column 1, otherwise we just have to check to see that we're not
		on line 1.
	*/
	
	if( (numcols && (getlnum()>1 || getcurrcol()!=1))
		|| (!numcols && getlnum()>1) )
		do_new_pg();
	
	/*	Now we've done a new page, or we were already at the start of a page.
		If the e/o modifier was used we may need to start another one.
	*/
	
	switch(f_toupper(c=getkey()))
	{
		case 'E':
			if(pageodd()) do_new_pg();
			break;
		
		case 'O':
			if(!pageodd()) do_new_pg();
			break;
		
		default:
			returnkey(c);
	}
}

/*	============================================================
	This actually starts a new page. It simply sends blank lines
	until the bottom margin is encountered ... then it has to print
	the bottom margin!
*/

void
do_new_pg(void)
{
	int oldcols=numcols;
	
	if(getlnum()<gettopmarg())				 /* do top margin, if needed */
	{
		numcols=0;
		dotopmargin();
		numcols=oldcols;
	}
	
	while(!at_page_bot()) blankline();			/* skip text part of page */
	
	if(numcols) flushcols();			  /* flush column buffer for body */
	
	numcols=0;
	dobotmargin();									  /* do bottom margin */
	
	docodes(pb);								   /* pb codes, if defined */
	
	/*	Now at the top of the next page... we need to inc the page number,
		do the top of page WHEN and  set the printing flags
	*/
	
	setnumval('p', getnumval('p')+1);				/* increment page num. */
	currline=0;
	
	setprintflag();							 /* see if we print next page */
	pausepage();								 /* do pause if necessary */
	
	numcols=oldcols;
}

/*	============================================================
	Set the print-this-page flag. If 'printpg' is 0 then the page
	is not printed.
	
	This is called by the new page routine and init.
*/

void
setprintflag(void)
{
	u_int currpage=getnumval('P');
	int t;
	
	printpg=0;  /* assume we don't want to print this page... */
	
	if(printoff) printpg=0;
	
	/*	Check to see if page ranges were specified. If they were go through
		them and see if the current page is within any of the ranges.
	*/
	
	else if(numranges)
	{
		for(t=printpg=0; t<numranges; t++)
		{
			if(currpage>=ranges[t].start && currpage<=ranges[t].end)
			{
				printpg=1;
				break;
			}
		}
	}
	else printpg=1;  /* no ranges, do this page! */
	
	if(printpg && oe_range)
	{
		if(oe_range==1 && !(currpage & 1)) printpg=0;
		else if(oe_range==2 && (currpage & 1)) printpg=0;
	}
}


/*	============================================================
	The page bottom codes are sent at the bottom of each page if
	defined. This can be useful to keep paging in alignment when
	doing non-standard line spacing, etc.
*/

void
setpbcodes(void)
{
	skipequal();
	
	setcodes(&pb,"page bottom");
}
