/* state.c */

#include "vpt.h"
#include "att_table.h"

#define MAXSTATES 10

static int statecount;

static char *state_buffs[MAXSTATES];


void
statesave(void)
{
	int buffsize;
	char *p;
	u_int ids[6];
	int numsize, tabsize, fmtsize;
	char *numstart,  *tabstart, *fmtstart;
	Att_table *bodyatts;
	
	if(statecount>=MAXSTATES-1)
		terminate("Only 10 STATE saves permitted");		   
	getnumstate(&numsize, &numstart);
	gettabstate(&tabsize, &tabstart);
	getfmtstate(&fmtsize, &fmtstart);
	bodyatts=getbodyatts();
	
	ids[0]=getbothid();
	ids[1]=getleftid();
	ids[2]=getrightid();
	ids[3]=linespace;
	ids[4]=halfline_on;
	ids[5]=partline_space;
	
	buffsize=sizeof(ids)+sizeof(Att_table)+numsize+
		tabsize+fmtsize+sizeof(tabfill);
	
	if((p=malloc(buffsize))==NULL) terminate("State buffer memory overflow");
	state_buffs[statecount]=p;
	
	memcpy(p, ids, sizeof(ids));
	p+=sizeof(ids);
	
	memcpy(p, bodyatts, sizeof(Att_table));
	p+=sizeof(Att_table);
	
	memcpy(p, numstart, numsize);
	p+=numsize;
	
	memcpy(p, tabstart, tabsize);
	p+=tabsize;
	
	memcpy(p, fmtstart, fmtsize);
	p+=fmtsize;
	
	memcpy(p, &tabfill, sizeof(tabfill));
	
	statecount++;
}


void
staterest(void)
{
	char *p;
	u_int ids[6];
	int numsize, tabsize, fmtsize;
	char *numstart, *tabstart, *fmtstart;
	Att_table *bodyatts;
	
	if(!statecount) error("No STATEs saved to restore");
	
	getnumstate(&numsize, &numstart);
	gettabstate(&tabsize, &tabstart);
	getfmtstate(&fmtsize, &fmtstart);
	bodyatts=getbodyatts();
	
	p=state_buffs[--statecount];
	
	memcpy(ids, p, sizeof(ids));
	p+=sizeof(ids);
	
	restbothid(ids[0]);
	restleftid(ids[1]);
	restrightid(ids[2]);
	linespace=ids[3];
	
	if(halfline_on!=ids[4] || partline_space!=ids[5]) fix_partline();
	
	halfline_on=ids[4];
	partline_space=ids[5];
	
	memcpy(bodyatts, p, sizeof(Att_table));
	p+=sizeof(Att_table);
	
	memcpy(numstart, p, numsize);
	p+=numsize;
	
	memcpy(tabstart, p, tabsize);
	p+=tabsize;
	
	memcpy(fmtstart, p, fmtsize);
	p+=fmtsize;
	
	memcpy(&tabfill, p, sizeof(tabfill));
	
	free(state_buffs[statecount]);
}


/*	======================================================================
	Ensure no state are active if changing the MS value
*/

void
ms_state(u_int n)
{
	if(statecount) terminate("MS=%d follows a save state command",n);
}

