/* when.c */

#include "vpt.h"

/* there are 11 when buffers. 0..9 and T. */

#define MAXWHENSIZE 160

static char whenbuff[10][MAXWHENSIZE];
static char whentbuff[MAXWHENSIZE];
static char doingwhen;

static char *pointwhen(char c);


/*	============================================================
	Point to a when buffer.
*/

static char *
pointwhen(char c)
{
	if(c=='t' || c=='T') return whentbuff;
	if(c>='0' && c<='9') return whenbuff[c-'0'];
	error("Unknown WHEN buffer");
	
	return NULL;	/* unreachable */
}


/*  ============================================================
	Define a when. WHENs are invoked after every CR is sent. They are
	pushed onto the returned key buffer, so they have priority over
	outstanding macros, etc.
	
	Valid buffers are 0..9 and T.
	
	0..9 are checked after every line, T is done at the top of each page.
	
*/

void
def_when(void)
{
	char *p, c;
	
	p=pointwhen(getkey());  /* point to when buffer 0..9, t */
	
	skipequal();
	*p=0;  /* start off by undefining  */
	
	if((c=getkey())!='!')
	{
		returnkey(c);
		getdstring(p, MAXWHENSIZE-4, "WHEN data");  /* get data */
	}
}

/*  ============================================================
	Process the end of line whens
*/

void
doeolwhens(void)
{
	char c;
	
	for(c='0'; c<='9'; dowhen(c++));  /* do all 10 whens */
}

/*  ============================================================
	Process a when.
	
	This is called from format and new page routines. The when buffer
	to process is specified.
*/

void
dowhen(char n)
{
	char *p=pointwhen(n);
	int m;
	KCHAR c;
	
	if(*p && !doingwhen)
	{
		doingwhen++;
		pushkbuff();
		m=addmacro(p);
		
		/* Now we loop until the end of the macro. A macro is terminated
			with a INT_DELIM. We assume that everything in the when IS a
			command...
		*/
		
		while(1)
		{
			docmdline();						/* process it				*/
			if((c=getnonb())==INT_DELIM) break;	/* end of macro, exit		*/
			else returnkey(c);					/* more stuff to do...loop	*/
		}
		pullkbuff();
		killmacro(m);
		doingwhen--;
	}
}
